@@ -625,7 +625,6 @@ typedef struct _array {
 #define ArrayValueSet(a,i,v) (BoxValueSet(ArrayValueBox(a,i),ArrayValueElt(a,i), v))
 
 typedef struct _io_chain {
-    DataType		*data;
     struct _io_chain	*next;
     int			size;
     int			used;
@@ -1036,6 +1035,7 @@ Type	*BuildArrayType (Type *type, int ndim, ...);
 Value	BinaryOperate (Value av, Value bv, BinaryOp operator);
 Value	UnaryOperate (Value v, UnaryOp operator);
 Value	NumericDiv (Value av, Value bv, int expandOk);
+Value	NumericMod (Value av, Value bv, int expandOk);
 
 # define	OK_TRUNC	1
 
